package PCC;
import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;

/**
 * Implementao de Table Model para exibir os Scios.
 * 
 * @author Eric Yuzo
 */
public class VendaTableModel extends AbstractTableModel {

	private static final long serialVersionUID = 1L;

	/* Lista de Scios que representam as linhas. */
	private List<ItemVenda> linhas;

	/* Array de Strings com o nome das colunas. */
	private String[] colunas = new String[] {
			"Produto", "Preo", "Quantidade", "Total"};


	public List<ItemVenda> getItensVenda(){
		return linhas;
	}
	
	/* Cria um SocioTableModel vazio. */
	public VendaTableModel() {
		linhas = new ArrayList<ItemVenda>();
	}

	/* Cria um SocioTableModel carregado com
	 * a lista de scios especificada. */
	public VendaTableModel(List<ItemVenda> listaDeSocios) {
		linhas = new ArrayList<ItemVenda>(listaDeSocios);
	}


	/* Retorna a quantidade de colunas. */
	@Override
	public int getColumnCount() {
		// Est retornando o tamanho do array "colunas".
		// Mas como o array  fixo, vai sempre retornar 4.
		return colunas.length;
	}
	
	/* Retorna a quantidade de linhas. */
	@Override
	public int getRowCount() {
		// Retorna o tamanho da lista de scios.
		return linhas.size();
	}

	/* Retorna o nome da coluna no ndice especificado.
	 * Este mtodo  usado pela JTable para saber o texto do cabealho. */
	@Override
	public String getColumnName(int columnIndex) {
		// Retorna o contedo do Array que possui o nome das colunas
		// no ndice especificado.
		return colunas[columnIndex];
	};

	/* Retorna a classe dos elementos da coluna especificada.
	 * Este mtodo  usado pela JTable na hora de definir o editor da clula.
	 * Como todos os campos deste model so Strings (nome, e-mail e fones),
	 * o retorno ser sempre a classe String. */
	@Override
	public Class<?> getColumnClass(int columnIndex) {
		//return String.class;
		
		switch (columnIndex) {
		case 0: // Primeira coluna  o nome.
			return String.class;
		case 1: // Segunda coluna  o e-mail.
			return Double.class;
		case 2: // Terceira coluna  o telefone fixo.
			return Integer.class;
		case 3: // Quarta coluna  o telefone celular.
			return Double.class;
		default:
			// Se o ndice da coluna no for vlido, lana um
			// IndexOutOfBoundsException (Exceo de ndice fora dos limites).
			// No foi necessrio verificar se o ndice da linha  invlido,
			// pois o prprio ArrayList lana a exceo caso seja invlido.
			throw new IndexOutOfBoundsException("columnIndex out of bounds");
		}
		
		
	};

	/* Retorna o valor da clula especificada
	 * pelos ndices da linha e da coluna. */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		// Pega o scio da linha especificada.
		ItemVenda itemVenda = linhas.get(rowIndex);

		// Retorna o campo referente a coluna especificada.
		// Aqui  feito um switch para verificar qual  a coluna
		// e retornar o campo adequado. As colunas so as mesmas
		// que foram especificadas no array "colunas".
		switch (columnIndex) {
		case 0: // Primeira coluna  o nome.
			return itemVenda.getProduto();
		case 1: // Segunda coluna  o e-mail.
			return itemVenda.getPreco();
		case 2: // Terceira coluna  o telefone fixo.
			return itemVenda.getQtdade();
		case 3: // Quarta coluna  o telefone celular.
			return itemVenda.getTotalItem();
		default:
			// Se o ndice da coluna no for vlido, lana um
			// IndexOutOfBoundsException (Exceo de ndice fora dos limites).
			// No foi necessrio verificar se o ndice da linha  invlido,
			// pois o prprio ArrayList lana a exceo caso seja invlido.
			throw new IndexOutOfBoundsException("columnIndex out of bounds");
		}
	}

	/* Seta o valor da clula especificada
	 * pelos ndices da linha e da coluna.
	 * Aqui ele est implementado para no fazer nada,
	 * at porque este table model no  editvel. */
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {};

	/* Retorna um valor booleano que define se a clula em questo
	 * pode ser editada ou no.
	 * Este mtodo  utilizado pela JTable na hora de definir o editor da clula.
	 * Neste caso, estar sempre retornando false, no permitindo que nenhuma
	 * clula seja editada. */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return false;
	}


	////////////////////////////////////////////////////////////
	// Os mtodos declarados at aqui foram as implementaes //
	// de TableModel, que so continuamente utilizados        //
	// pela JTable para definir seu comportamento,            //
	// por isso o nome Table Model (Modelo da Tabela).        //
	//													      //
	// A partir de agora, os mtodos criados sero            //
	// particulares desta classe. Eles sero teis            //
	// em algumas situaes.							      //
	////////////////////////////////////////////////////////////


	/* Retorna o scio da linha especificada. */
	public void getSocio(int indiceLinha) {
		linhas.get(indiceLinha);
	}
	
	/* Adiciona um registro. */
	public void addItemVenda(ItemVenda item) {
		// Adiciona o registro.
		linhas.add(item);

		// Pega a quantidade de registros e subtrai um para achar
		// o ltimo ndice.  preciso subtrair um, pois os ndices
		// comeam pelo zero.
		int ultimoIndice = getRowCount() - 1;

		// Reporta a mudana. O JTable recebe a notificao
		// e se redesenha permitindo que visualizemos a atualizao.
		fireTableRowsInserted(ultimoIndice, ultimoIndice);
	}

	/* Remove a linha especificada. */
	public void removeSocio(int indiceLinha) {
		// Remove o scio da linha especificada.    	
		linhas.remove(indiceLinha);

		// Reporta a mudana. O JTable recebe a notificao
		// e se redesenha permitindo que visualizemos a atualizao.
		fireTableRowsDeleted(indiceLinha, indiceLinha);
	}


	/* Remove todos os registros. */
	public void limpar() {
		// Remove todos os elementos da lista de scios.
		linhas.clear();

		// Reporta a mudana. O JTable recebe a notificao
		// e se redesenha permitindo que visualizemos a atualizao.
		fireTableDataChanged();
	}

	/* Verifica se este table model est vazio. */
	public boolean isEmpty() {
		return linhas.isEmpty();
	}

}